"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RequirementID(str, Enum):
    r"""The unique ID of what the requirement is asking to be filled out."""

    ACCOUNT_TOS_ACCEPTANCE = "account.tos-acceptance"
    INDIVIDUAL_MOBILE = "individual.mobile"
    INDIVIDUAL_EMAIL = "individual.email"
    INDIVIDUAL_EMAIL_OR_MOBILE = "individual.email-or-mobile"
    INDIVIDUAL_FIRSTNAME = "individual.firstname"
    INDIVIDUAL_LASTNAME = "individual.lastname"
    INDIVIDUAL_ADDRESS = "individual.address"
    INDIVIDUAL_SSN_LAST4 = "individual.ssn-last4"
    INDIVIDUAL_SSN = "individual.ssn"
    INDIVIDUAL_BIRTHDATE = "individual.birthdate"
    BUSINESS_LEGALNAME = "business.legalname"
    BUSINESS_DESCRIPTION_OR_WEBSITE = "business.description-or-website"
    BUSINESS_ENTITY_TYPE = "business.entity-type"
    BUSINESS_BUSINESS_TYPE = "business.business-type"
    BUSINESS_DBA = "business.dba"
    BUSINESS_EIN = "business.ein"
    BUSINESS_ADDRESS = "business.address"
    BUSINESS_PHONE = "business.phone"
    BUSINESS_ADMINS = "business.admins"
    BUSINESS_CONTROLLERS = "business.controllers"
    BUSINESS_OWNERS = "business.owners"
    BUSINESS_CLASSIFICATION = "business.classification"
    BUSINESS_INDUSTRY_CODE_MCC = "business.industry-code-mcc"
    BUSINESS_INDUSTRY = "business.industry"
    BUSINESS_INDICATE_OWNERS_PROVIDED = "business.indicate-owners-provided"
    BUSINESS_AVERAGE_TRANSACTION_SIZE = "business.average-transaction-size"
    BUSINESS_MAX_TRANSACTION_SIZE = "business.max-transaction-size"
    BUSINESS_AVERAGE_MONTHLY_TRANSACTION_VOLUME = (
        "business.average-monthly-transaction-volume"
    )
    BUSINESS_VOLUME_BY_CUSTOMER_TYPE = "business.volume-by-customer-type"
    BUSINESS_CARD_VOLUME_DISTRIBUTION = "business.card-volume-distribution"
    BUSINESS_FULFILLMENT = "business.fulfillment"
    BUSINESS_DESCRIPTION = "business.description"
    BUSINESS_UNDERWRITING_DOCUMENTS_TIER_ONE = (
        "business.underwriting-documents-tier-one"
    )
    BANK_ACCOUNTS_NAME = "bank-accounts.name"
    BANK_ACCOUNTS_ROUTING_NUMBER = "bank-accounts.routing-number"
    BANK_ACCOUNTS_ACCOUNT_NUMBER = "bank-accounts.account-number"
    DOCUMENT_BUSINESS_VERIFICATION = "document.business.verification"
    DOCUMENT_INDIVIDUAL_TIN = "document.individual.tin"
    DOCUMENT_INDIVIDUAL_VERIFICATION = "document.individual.verification"
    DOCUMENT_REPRESENTATIVE_REP_UUID_TIN = "document.representative.{rep-uuid}.tin"
    DOCUMENT_REPRESENTATIVE_REP_UUID_VERIFICATION = (
        "document.representative.{rep-uuid}.verification"
    )
    REPRESENTATIVE_REP_UUID_MOBILE = "representative.{rep-uuid}.mobile"
    REPRESENTATIVE_REP_UUID_EMAIL = "representative.{rep-uuid}.email"
    REPRESENTATIVE_REP_UUID_EMAIL_OR_MOBILE = (
        "representative.{rep-uuid}.email-or-mobile"
    )
    REPRESENTATIVE_REP_UUID_FIRSTNAME = "representative.{rep-uuid}.firstname"
    REPRESENTATIVE_REP_UUID_LASTNAME = "representative.{rep-uuid}.lastname"
    REPRESENTATIVE_REP_UUID_ADDRESS = "representative.{rep-uuid}.address"
    REPRESENTATIVE_REP_UUID_SSN_LAST4 = "representative.{rep-uuid}.ssn-last4"
    REPRESENTATIVE_REP_UUID_SSN = "representative.{rep-uuid}.ssn"
    REPRESENTATIVE_REP_UUID_BIRTHDATE = "representative.{rep-uuid}.birthdate"
    REPRESENTATIVE_REP_UUID_JOB_TITLE = "representative.{rep-uuid}.job-title"
    REPRESENTATIVE_REP_UUID_IS_CONTROLLER = "representative.{rep-uuid}.is-controller"
    REPRESENTATIVE_REP_UUID_IS_OWNER = "representative.{rep-uuid}.is-owner"
    REPRESENTATIVE_REP_UUID_OWNERSHIP = "representative.{rep-uuid}.ownership"
    DOCUMENT_DOC_UUID_ = "document.{doc-uuid}"
    UNDERWRITING_BUSINESS_PRESENCE = "underwriting.businessPresence"
    UNDERWRITING_GEOGRAPHIC_REACH = "underwriting.geographicReach"
    UNDERWRITING_VOLUME_SHARE_BY_CUSTOMER_TYPE = (
        "underwriting.volumeShareByCustomerType"
    )
    UNDERWRITING_PENDING_LITIGATION = "underwriting.pendingLitigation"
    UNDERWRITING_DOCUMENTS_PROCESSING_STATEMENTS_PAST3_MONTHS = (
        "underwriting.documents.processingStatementsPast3Months"
    )
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_CARD_ACCEPTANCE_METHODS = (
        "underwriting.collectFunds.cardPayments.cardAcceptanceMethods"
    )
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.collectFunds.cardPayments.currentlyAcceptsCards"
    )
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = "underwriting.collectFunds.cardPayments.estimatedActivity.averageTransactionAmount"
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = "underwriting.collectFunds.cardPayments.estimatedActivity.maximumTransactionAmount"
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.collectFunds.cardPayments.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_FULFILLMENT = (
        "underwriting.collectFunds.cardPayments.fulfillment"
    )
    UNDERWRITING_COLLECT_FUNDS_CARD_PAYMENTS_REFUND_POLICY = (
        "underwriting.collectFunds.cardPayments.refundPolicy"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_CARD_ACCEPTANCE_METHODS = (
        "underwriting.collectFunds.ach.cardAcceptanceMethods"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.collectFunds.ach.currentlyAcceptsCards"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = (
        "underwriting.collectFunds.ach.estimatedActivity.averageTransactionAmount"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = (
        "underwriting.collectFunds.ach.estimatedActivity.maximumTransactionAmount"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.collectFunds.ach.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_FULFILLMENT = (
        "underwriting.collectFunds.ach.fulfillment"
    )
    UNDERWRITING_COLLECT_FUNDS_ACH_REFUND_POLICY = (
        "underwriting.collectFunds.ach.refundPolicy"
    )
    UNDERWRITING_SEND_FUNDS_ACH_CARD_ACCEPTANCE_METHODS = (
        "underwriting.sendFunds.ach.cardAcceptanceMethods"
    )
    UNDERWRITING_SEND_FUNDS_ACH_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.sendFunds.ach.currentlyAcceptsCards"
    )
    UNDERWRITING_SEND_FUNDS_ACH_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = (
        "underwriting.sendFunds.ach.estimatedActivity.averageTransactionAmount"
    )
    UNDERWRITING_SEND_FUNDS_ACH_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = (
        "underwriting.sendFunds.ach.estimatedActivity.maximumTransactionAmount"
    )
    UNDERWRITING_SEND_FUNDS_ACH_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.sendFunds.ach.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_SEND_FUNDS_ACH_FULFILLMENT = "underwriting.sendFunds.ach.fulfillment"
    UNDERWRITING_SEND_FUNDS_ACH_REFUND_POLICY = (
        "underwriting.sendFunds.ach.refundPolicy"
    )
    UNDERWRITING_SEND_FUNDS_RTP_CARD_ACCEPTANCE_METHODS = (
        "underwriting.sendFunds.rtp.cardAcceptanceMethods"
    )
    UNDERWRITING_SEND_FUNDS_RTP_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.sendFunds.rtp.currentlyAcceptsCards"
    )
    UNDERWRITING_SEND_FUNDS_RTP_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = (
        "underwriting.sendFunds.rtp.estimatedActivity.averageTransactionAmount"
    )
    UNDERWRITING_SEND_FUNDS_RTP_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = (
        "underwriting.sendFunds.rtp.estimatedActivity.maximumTransactionAmount"
    )
    UNDERWRITING_SEND_FUNDS_RTP_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.sendFunds.rtp.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_SEND_FUNDS_RTP_FULFILLMENT = "underwriting.sendFunds.rtp.fulfillment"
    UNDERWRITING_SEND_FUNDS_RTP_REFUND_POLICY = (
        "underwriting.sendFunds.rtp.refundPolicy"
    )
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_CARD_ACCEPTANCE_METHODS = (
        "underwriting.sendFunds.pushToCard.cardAcceptanceMethods"
    )
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.sendFunds.pushToCard.currentlyAcceptsCards"
    )
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = "underwriting.sendFunds.pushToCard.estimatedActivity.averageTransactionAmount"
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = "underwriting.sendFunds.pushToCard.estimatedActivity.maximumTransactionAmount"
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.sendFunds.pushToCard.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_FULFILLMENT = (
        "underwriting.sendFunds.pushToCard.fulfillment"
    )
    UNDERWRITING_SEND_FUNDS_PUSH_TO_CARD_REFUND_POLICY = (
        "underwriting.sendFunds.pushToCard.refundPolicy"
    )
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_CARD_ACCEPTANCE_METHODS = (
        "underwriting.moneyTransfer.pullFromCard.cardAcceptanceMethods"
    )
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.moneyTransfer.pullFromCard.currentlyAcceptsCards"
    )
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = "underwriting.moneyTransfer.pullFromCard.estimatedActivity.averageTransactionAmount"
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = "underwriting.moneyTransfer.pullFromCard.estimatedActivity.maximumTransactionAmount"
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = "underwriting.moneyTransfer.pullFromCard.estimatedActivity.monthlyVolumeRange"
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_FULFILLMENT = (
        "underwriting.moneyTransfer.pullFromCard.fulfillment"
    )
    UNDERWRITING_MONEY_TRANSFER_PULL_FROM_CARD_REFUND_POLICY = (
        "underwriting.moneyTransfer.pullFromCard.refundPolicy"
    )
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_CARD_ACCEPTANCE_METHODS = (
        "underwriting.moneyTransfer.pushToCard.cardAcceptanceMethods"
    )
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_CURRENTLY_ACCEPTS_CARDS = (
        "underwriting.moneyTransfer.pushToCard.currentlyAcceptsCards"
    )
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_ESTIMATED_ACTIVITY_AVERAGE_TRANSACTION_AMOUNT = "underwriting.moneyTransfer.pushToCard.estimatedActivity.averageTransactionAmount"
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_ESTIMATED_ACTIVITY_MAXIMUM_TRANSACTION_AMOUNT = "underwriting.moneyTransfer.pushToCard.estimatedActivity.maximumTransactionAmount"
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_ESTIMATED_ACTIVITY_MONTHLY_VOLUME_RANGE = (
        "underwriting.moneyTransfer.pushToCard.estimatedActivity.monthlyVolumeRange"
    )
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_FULFILLMENT = (
        "underwriting.moneyTransfer.pushToCard.fulfillment"
    )
    UNDERWRITING_MONEY_TRANSFER_PUSH_TO_CARD_REFUND_POLICY = (
        "underwriting.moneyTransfer.pushToCard.refundPolicy"
    )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amountdecimal import AmountDecimal, AmountDecimalTypedDict
from .transferparty import TransferParty
from moovio_sdk.types import BaseModel
import pydantic
from typing import List
from typing_extensions import Annotated, TypedDict


class MoovFeeTypedDict(TypedDict):
    r"""Moov fee charged to an account involved in a transfer."""

    account_id: str
    r"""ID of the account that fees were charged to."""
    transfer_party: TransferParty
    r"""Indicates whether the account charged was the partner, source, or destination of the transfer."""
    total_amount: AmountDecimalTypedDict
    r"""The total amount of fees charged to the account."""
    fee_i_ds: List[str]
    r"""List of fee IDs that sum to the totalAmount."""


class MoovFee(BaseModel):
    r"""Moov fee charged to an account involved in a transfer."""

    account_id: Annotated[str, pydantic.Field(alias="accountID")]
    r"""ID of the account that fees were charged to."""

    transfer_party: Annotated[TransferParty, pydantic.Field(alias="transferParty")]
    r"""Indicates whether the account charged was the partner, source, or destination of the transfer."""

    total_amount: Annotated[AmountDecimal, pydantic.Field(alias="totalAmount")]
    r"""The total amount of fees charged to the account."""

    fee_i_ds: Annotated[List[str], pydantic.Field(alias="feeIDs")]
    r"""List of fee IDs that sum to the totalAmount."""


try:
    MoovFee.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel
from typing_extensions import TypedDict


class CreateReversalTypedDict(TypedDict):
    amount: int
    r"""Amount to reverse in cents. Partial amounts will automatically trigger a refund instead of a cancellation."""


class CreateReversal(BaseModel):
    amount: int
    r"""Amount to reverse in cents. Partial amounts will automatically trigger a refund instead of a cancellation."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .terminalapplicationplatform import TerminalApplicationPlatform
from .terminalapplicationstatus import TerminalApplicationStatus
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class TerminalApplicationTypedDict(TypedDict):
    r"""Describes a terminal application."""

    terminal_application_id: str
    r"""ID of the terminal application."""
    status: TerminalApplicationStatus
    r"""Status of the terminal application."""
    platform: TerminalApplicationPlatform
    r"""Platform of the terminal application."""
    app_bundle_id: NotRequired[str]
    r"""The app bundle identifier of the terminal application. Will be returned if platform is `ios`."""
    package_name: NotRequired[str]
    r"""The app package name of the terminal application. Will be returned if platform is `android`."""
    sha256_digest: NotRequired[str]
    r"""A cryptographic hash of the signing key for the application. Will be returned if platform is `android`."""
    version_code: NotRequired[str]
    r"""The app version code of the terminal application. Will be returned if platform is `android`."""


class TerminalApplication(BaseModel):
    r"""Describes a terminal application."""

    terminal_application_id: Annotated[
        str, pydantic.Field(alias="terminalApplicationID")
    ]
    r"""ID of the terminal application."""

    status: TerminalApplicationStatus
    r"""Status of the terminal application."""

    platform: TerminalApplicationPlatform
    r"""Platform of the terminal application."""

    app_bundle_id: Annotated[Optional[str], pydantic.Field(alias="appBundleID")] = None
    r"""The app bundle identifier of the terminal application. Will be returned if platform is `ios`."""

    package_name: Annotated[Optional[str], pydantic.Field(alias="packageName")] = None
    r"""The app package name of the terminal application. Will be returned if platform is `android`."""

    sha256_digest: Annotated[Optional[str], pydantic.Field(alias="sha256Digest")] = None
    r"""A cryptographic hash of the signing key for the application. Will be returned if platform is `android`."""

    version_code: Annotated[Optional[str], pydantic.Field(alias="versionCode")] = None
    r"""The app version code of the terminal application. Will be returned if platform is `android`."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["appBundleID", "packageName", "sha256Digest", "versionCode"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    TerminalApplication.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class BusinessPresence(str, Enum):
    COMMERCIAL_OFFICE = "commercial-office"
    HOME_BASED = "home-based"
    MIXED_PRESENCE = "mixed-presence"
    MOBILE_BUSINESS = "mobile-business"
    ONLINE_ONLY = "online-only"
    RETAIL_STOREFRONT = "retail-storefront"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class OccurrenceStatus(str, Enum):
    r"""Status of the completed occurrence."""

    PENDING = "pending"
    FAILED = "failed"
    COMPLETED = "completed"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from enum import Enum
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional


class GetAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    APPLICATION_PDF = "application/pdf"


class Statements(BaseSDK):
    def list(
        self,
        *,
        account_id: str,
        billing_period_start_date_time: Optional[datetime] = None,
        billing_period_end_date_time: Optional[datetime] = None,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListStatementsResponse:
        r"""Retrieve all statements associated with an account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param billing_period_start_date_time: Optional date-time which inclusively filters all statements where billing period is on or after this date-time.
        :param billing_period_end_date_time: Optional date-time which exclusively filters all statements where billing period is before this date-time.
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListStatementsRequest(
            billing_period_start_date_time=billing_period_start_date_time,
            billing_period_end_date_time=billing_period_end_date_time,
            skip=skip,
            count=count,
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/statements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListStatementsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listStatements",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListStatementsResponse(
                result=unmarshal_json_response(List[components.Statement], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        account_id: str,
        billing_period_start_date_time: Optional[datetime] = None,
        billing_period_end_date_time: Optional[datetime] = None,
        skip: Optional[int] = None,
        count: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.ListStatementsResponse:
        r"""Retrieve all statements associated with an account.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param billing_period_start_date_time: Optional date-time which inclusively filters all statements where billing period is on or after this date-time.
        :param billing_period_end_date_time: Optional date-time which exclusively filters all statements where billing period is before this date-time.
        :param skip:
        :param count:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListStatementsRequest(
            billing_period_start_date_time=billing_period_start_date_time,
            billing_period_end_date_time=billing_period_end_date_time,
            skip=skip,
            count=count,
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/statements",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.ListStatementsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listStatements",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "403", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.ListStatementsResponse(
                result=unmarshal_json_response(List[components.Statement], http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        account_id: str,
        statement_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GetAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetStatementResponse:
        r"""Retrieve a statement by its ID.

        Use the `Accept` header to specify the format of the response. Supported formats are `application/json` and `application/pdf`.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param statement_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetStatementRequest(
            account_id=account_id,
            statement_id=statement_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/statements/{statementID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, application/pdf;q=0",
            http_headers=http_headers,
            _globals=operations.GetStatementGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getStatement",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return operations.GetStatementResponse(
                result=unmarshal_json_response(
                    components.Statement, http_res, http_res_text
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "200", "application/pdf"):
            return operations.GetStatementResponse(
                result=http_res, headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        http_res_text = utils.stream_to_text(http_res)
        raise errors.APIError("Unexpected response received", http_res, http_res_text)

    async def get_async(
        self,
        *,
        account_id: str,
        statement_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GetAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetStatementResponse:
        r"""Retrieve a statement by its ID.

        Use the `Accept` header to specify the format of the response. Supported formats are `application/json` and `application/pdf`.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/profile.read` scope.

        :param account_id:
        :param statement_id:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetStatementRequest(
            account_id=account_id,
            statement_id=statement_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/statements/{statementID}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, application/pdf;q=0",
            http_headers=http_headers,
            _globals=operations.GetStatementGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getStatement",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            stream=True,
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return operations.GetStatementResponse(
                result=unmarshal_json_response(
                    components.Statement, http_res, http_res_text
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "200", "application/pdf"):
            return operations.GetStatementResponse(
                result=http_res, headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.APIError("Unexpected response received", http_res, http_res_text)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    end2endencryptionerror as components_end2endencryptionerror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class UpdateCardErrorData(BaseModel):
    e2ee: Optional[components_end2endencryptionerror.End2EndEncryptionError] = None
    billing_address: Annotated[
        Optional[str], pydantic.Field(alias="billingAddress")
    ] = None
    expiration: Optional[str] = None
    card_cvv: Annotated[Optional[str], pydantic.Field(alias="cardCvv")] = None
    card_on_file: Annotated[Optional[str], pydantic.Field(alias="cardOnFile")] = None
    merchant_account_id: Annotated[
        Optional[str], pydantic.Field(alias="merchantAccountID")
    ] = None
    verify_name: Annotated[Optional[str], pydantic.Field(alias="verifyName")] = None
    holder_name: Annotated[Optional[str], pydantic.Field(alias="holderName")] = None


@dataclass(unsafe_hash=True)
class UpdateCardError(MoovError):
    data: UpdateCardErrorData = field(hash=False)

    def __init__(
        self,
        data: UpdateCardErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .address import Address, AddressTypedDict
from .birthdate import BirthDate, BirthDateTypedDict
from .governmentid import GovernmentID, GovernmentIDTypedDict
from .individualname import IndividualName, IndividualNameTypedDict
from .phonenumber import PhoneNumber, PhoneNumberTypedDict
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateIndividualProfileTypedDict(TypedDict):
    name: IndividualNameTypedDict
    phone: NotRequired[PhoneNumberTypedDict]
    email: NotRequired[str]
    address: NotRequired[AddressTypedDict]
    birth_date: NotRequired[BirthDateTypedDict]
    government_id: NotRequired[GovernmentIDTypedDict]


class CreateIndividualProfile(BaseModel):
    name: IndividualName

    phone: Optional[PhoneNumber] = None

    email: Optional[str] = None

    address: Optional[Address] = None

    birth_date: Annotated[Optional[BirthDate], pydantic.Field(alias="birthDate")] = None

    government_id: Annotated[
        Optional[GovernmentID], pydantic.Field(alias="governmentID")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            ["phone", "email", "address", "birthDate", "governmentID"]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    CreateIndividualProfile.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .returnpolicytype import ReturnPolicyType
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class FulfillmentDetailsTypedDict(TypedDict):
    has_physical_goods: bool
    is_shipping_product: bool
    shipment_duration_days: int
    return_policy: ReturnPolicyType


class FulfillmentDetails(BaseModel):
    has_physical_goods: Annotated[bool, pydantic.Field(alias="hasPhysicalGoods")]

    is_shipping_product: Annotated[bool, pydantic.Field(alias="isShippingProduct")]

    shipment_duration_days: Annotated[int, pydantic.Field(alias="shipmentDurationDays")]

    return_policy: Annotated[ReturnPolicyType, pydantic.Field(alias="returnPolicy")]


try:
    FulfillmentDetails.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    amountvalidationerror as components_amountvalidationerror,
    createpaymentlinklineitemsvalidationerror as components_createpaymentlinklineitemsvalidationerror,
    displayoptionserror as components_displayoptionserror,
    paymentdetailserror as components_paymentdetailserror,
    payoutdetailserror as components_payoutdetailserror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class UpdatePaymentLinkErrorData(BaseModel):
    amount: Optional[components_amountvalidationerror.AmountValidationError] = None
    sales_tax_amount: Annotated[
        Optional[components_amountvalidationerror.AmountValidationError],
        pydantic.Field(alias="salesTaxAmount"),
    ] = None
    expires_on: Annotated[Optional[str], pydantic.Field(alias="expiresOn")] = None
    display: Optional[components_displayoptionserror.DisplayOptionsError] = None
    payment: Optional[components_paymentdetailserror.PaymentDetailsError] = None
    payout: Optional[components_payoutdetailserror.PayoutDetailsError] = None
    line_items: Annotated[
        Optional[
            components_createpaymentlinklineitemsvalidationerror.CreatePaymentLinkLineItemsValidationError
        ],
        pydantic.Field(alias="lineItems"),
    ] = None


@dataclass(unsafe_hash=True)
class UpdatePaymentLinkError(MoovError):
    data: UpdatePaymentLinkErrorData = field(hash=False)

    def __init__(
        self,
        data: UpdatePaymentLinkErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from datetime import datetime
from moovio_sdk.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class AsyncCreatedRefundTypedDict(TypedDict):
    r"""Asynchronous refund response"""

    refund_id: str
    created_on: datetime
    amount: AmountTypedDict


class AsyncCreatedRefund(BaseModel):
    r"""Asynchronous refund response"""

    refund_id: Annotated[str, pydantic.Field(alias="refundID")]

    created_on: Annotated[datetime, pydantic.Field(alias="createdOn")]

    amount: Amount


try:
    AsyncCreatedRefund.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class RequirementErrorCode(str, Enum):
    INVALID_VALUE = "invalid-value"
    FAILED_AUTOMATIC_VERIFICATION = "failed-automatic-verification"
    FAILED_OTHER = "failed-other"
    INVALID_ADDRESS = "invalid-address"
    ADDRESS_RESTRICTED = "address-restricted"
    TAX_ID_MISMATCH = "tax-id-mismatch"
    DOCUMENT_ID_MISMATCH = "document-id-mismatch"
    DOCUMENT_DATE_OF_BIRTH_MISMATCH = "document-date-of-birth-mismatch"
    DOCUMENT_NAME_MISMATCH = "document-name-mismatch"
    DOCUMENT_ADDRESS_MISMATCH = "document-address.mismatch"
    DOCUMENT_NUMBER_MISMATCH = "document-number-mismatch"
    DOCUMENT_INCOMPLETE = "document-incomplete"
    DOCUMENT_FAILED_RISK = "document-failed-risk"
    DOCUMENT_ILLEGIBLE = "document-illegible"
    DOCUMENT_UNSUPPORTED = "document-unsupported"
    DOCUMENT_NOT_UPLOADED = "document-not-uploaded"
    DOCUMENT_CORRUPT = "document-corrupt"
    DOCUMENT_EXPIRED = "document-expired"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class FeePlanAgreementStatus(str, Enum):
    ACTIVE = "active"
    TERMINATED = "terminated"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class PrimaryRegulator(str, Enum):
    r"""If the business is a financial institution, this field describes its primary regulator."""

    OCC = "OCC"
    FDIC = "FDIC"
    NCUA = "NCUA"
    FRB = "FRB"
    STATE_CU_REGULATOR = "state-cu-regulator"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.components import (
    createauthorizedusererror as components_createauthorizedusererror,
)
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated


class UpdateIssuedCardErrorData(BaseModel):
    state: Optional[str] = None
    memo: Optional[str] = None
    authorized_user: Annotated[
        Optional[components_createauthorizedusererror.CreateAuthorizedUserError],
        pydantic.Field(alias="authorizedUser"),
    ] = None


@dataclass(unsafe_hash=True)
class UpdateIssuedCardError(MoovError):
    data: UpdateIssuedCardErrorData = field(hash=False)

    def __init__(
        self,
        data: UpdateIssuedCardErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class E2EETokenUpdateTypedDict(TypedDict):
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """

    token: NotRequired[str]
    r"""An [RFC](https://datatracker.ietf.org/doc/html/rfc7516) compact-serialized JSON Web Encryption (JWE) token."""


class E2EETokenUpdate(BaseModel):
    r"""Wraps a compact-serialized JSON Web Encryption (JWE) token used for secure transmission of sensitive data (e.g., PCI information) through intermediaries.
    This token is encrypted using the public key from /end-to-end-keys and wraps an AES key. For details and examples, refer to our
    [GitHub repository](https://github.com/moovfinancial/moov-go/blob/main/examples/e2ee/e2ee_test.go).
    """

    token: Optional[str] = None
    r"""An [RFC](https://datatracker.ietf.org/doc/html/rfc7516) compact-serialized JSON Web Encryption (JWE) token."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["token"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from moovio_sdk import utils
from moovio_sdk._hooks import HookContext
from moovio_sdk.models import components, errors, operations
from moovio_sdk.types import OptionalNullable, UNSET
from moovio_sdk.utils import get_security_from_env
from moovio_sdk.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class ApplePay(BaseSDK):
    def register_merchant_domains(
        self,
        *,
        account_id: str,
        display_name: Optional[str] = None,
        domains: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.RegisterApplePayMerchantDomainsResponse:
        r"""Add domains to be registered with Apple Pay.

        Any domains that will be used to accept payments must first be [verified](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains)
        with Apple.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param display_name: A UTF-8 string to display in the Buy button.
        :param domains: A unique list of fully-qualified, top-level or sub-domain names where you will accept Apple Pay.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.RegisterApplePayMerchantDomainsRequest(
            account_id=account_id,
            register_apple_pay_merchant_domains=components.RegisterApplePayMerchantDomains(
                display_name=display_name,
                domains=domains,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.RegisterApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.register_apple_pay_merchant_domains,
                False,
                False,
                "json",
                components.RegisterApplePayMerchantDomains,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="registerApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "409",
                "417",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.RegisterApplePayMerchantDomainsResponse(
                result=unmarshal_json_response(
                    components.ApplePayMerchantDomains, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "409", "417", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def register_merchant_domains_async(
        self,
        *,
        account_id: str,
        display_name: Optional[str] = None,
        domains: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.RegisterApplePayMerchantDomainsResponse:
        r"""Add domains to be registered with Apple Pay.

        Any domains that will be used to accept payments must first be [verified](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains)
        with Apple.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param display_name: A UTF-8 string to display in the Buy button.
        :param domains: A unique list of fully-qualified, top-level or sub-domain names where you will accept Apple Pay.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.RegisterApplePayMerchantDomainsRequest(
            account_id=account_id,
            register_apple_pay_merchant_domains=components.RegisterApplePayMerchantDomains(
                display_name=display_name,
                domains=domains,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.RegisterApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.register_apple_pay_merchant_domains,
                False,
                False,
                "json",
                components.RegisterApplePayMerchantDomains,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="registerApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "409",
                "417",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.RegisterApplePayMerchantDomainsResponse(
                result=unmarshal_json_response(
                    components.ApplePayMerchantDomains, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "409", "417", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def update_merchant_domains(
        self,
        *,
        account_id: str,
        add_domains: Optional[List[str]] = None,
        remove_domains: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateApplePayMerchantDomainsResponse:
        r"""Add or remove domains to be registered with Apple Pay.

        Any domains that will be used to accept payments must first be [verified](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains)
        with Apple.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param add_domains: A unique list of fully-qualified, top-level or sub-domain names to add.
        :param remove_domains: A unique list of fully-qualified, top-level or sub-domain names to remove.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateApplePayMerchantDomainsRequest(
            account_id=account_id,
            update_apple_pay_merchant_domains=components.UpdateApplePayMerchantDomains(
                add_domains=add_domains,
                remove_domains=remove_domains,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_apple_pay_merchant_domains,
                False,
                False,
                "json",
                components.UpdateApplePayMerchantDomains,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "417",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.UpdateApplePayMerchantDomainsResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "409", "417", "429"], "*"
        ):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def update_merchant_domains_async(
        self,
        *,
        account_id: str,
        add_domains: Optional[List[str]] = None,
        remove_domains: Optional[List[str]] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateApplePayMerchantDomainsResponse:
        r"""Add or remove domains to be registered with Apple Pay.

        Any domains that will be used to accept payments must first be [verified](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains)
        with Apple.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param add_domains: A unique list of fully-qualified, top-level or sub-domain names to add.
        :param remove_domains: A unique list of fully-qualified, top-level or sub-domain names to remove.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateApplePayMerchantDomainsRequest(
            account_id=account_id,
            update_apple_pay_merchant_domains=components.UpdateApplePayMerchantDomains(
                add_domains=add_domains,
                remove_domains=remove_domains,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.UpdateApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.update_apple_pay_merchant_domains,
                False,
                False,
                "json",
                components.UpdateApplePayMerchantDomains,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "417",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "204", "*"):
            return operations.UpdateApplePayMerchantDomainsResponse(
                headers=utils.get_response_headers(http_res.headers)
            )
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(
            http_res, ["401", "403", "404", "409", "417", "429"], "*"
        ):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def get_merchant_domains(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetApplePayMerchantDomainsResponse:
        r"""Get domains registered with Apple Pay.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.read` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetApplePayMerchantDomainsRequest(
            account_id=account_id,
        )

        req = self._build_request(
            method="GET",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetApplePayMerchantDomainsResponse(
                result=unmarshal_json_response(
                    components.ApplePayMerchantDomains, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def get_merchant_domains_async(
        self,
        *,
        account_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetApplePayMerchantDomainsResponse:
        r"""Get domains registered with Apple Pay.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.read` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetApplePayMerchantDomainsRequest(
            account_id=account_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/accounts/{accountID}/apple-pay/domains",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.GetApplePayMerchantDomainsGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getApplePayMerchantDomains",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "403", "404", "429", "4XX", "500", "504", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return operations.GetApplePayMerchantDomainsResponse(
                result=unmarshal_json_response(
                    components.ApplePayMerchantDomains, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def create_session(
        self,
        *,
        account_id: str,
        domain: str,
        display_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateApplePaySessionResponse:
        r"""Create a session with Apple Pay to facilitate a payment.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.
        A successful response from this endpoint should be passed through to Apple Pay unchanged.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param domain: A fully qualified top-level or sub-domain name where you will accept Apple Pay. Should not include \"https\".
        :param display_name: A UTF-8 string to display in the Buy button.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateApplePaySessionRequest(
            account_id=account_id,
            create_apple_pay_session=components.CreateApplePaySession(
                domain=domain,
                display_name=display_name,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/apple-pay/sessions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateApplePaySessionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_apple_pay_session,
                False,
                False,
                "json",
                components.CreateApplePaySession,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createApplePaySession",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateApplePaySessionResponse(
                result=unmarshal_json_response(str, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409", "422"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def create_session_async(
        self,
        *,
        account_id: str,
        domain: str,
        display_name: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateApplePaySessionResponse:
        r"""Create a session with Apple Pay to facilitate a payment.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.
        A successful response from this endpoint should be passed through to Apple Pay unchanged.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/apple-pay.write` scope.

        :param account_id: ID of the Moov account representing the merchant.
        :param domain: A fully qualified top-level or sub-domain name where you will accept Apple Pay. Should not include \"https\".
        :param display_name: A UTF-8 string to display in the Buy button.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateApplePaySessionRequest(
            account_id=account_id,
            create_apple_pay_session=components.CreateApplePaySession(
                domain=domain,
                display_name=display_name,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/apple-pay/sessions",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.CreateApplePaySessionGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.create_apple_pay_session,
                False,
                False,
                "json",
                components.CreateApplePaySession,
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createApplePaySession",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.CreateApplePaySessionResponse(
                result=unmarshal_json_response(str, http_res),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409", "422"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    def link_token(
        self,
        *,
        account_id: str,
        token: Union[
            components.LinkApplePayToken, components.LinkApplePayTokenTypedDict
        ],
        billing_contact: Optional[
            Union[
                components.AppleBillingContact, components.AppleBillingContactTypedDict
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.LinkApplePayTokenResponse:
        r"""Connect an Apple Pay token to the specified account.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.
        The `token` data is defined by Apple Pay and should be passed through from Apple Pay's response unmodified.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id: ID of the Moov account representing the cardholder.
        :param token: Contains the user's payment information as returned from Apple Pay.

            Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken)
            for more information.
        :param billing_contact: Billing contact information as returned from Apple Pay.

            Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
            for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.LinkApplePayTokenRequest(
            account_id=account_id,
            link_apple_pay=components.LinkApplePay(
                token=utils.get_pydantic_model(token, components.LinkApplePayToken),
                billing_contact=utils.get_pydantic_model(
                    billing_contact, Optional[components.AppleBillingContact]
                ),
            ),
        )

        req = self._build_request(
            method="POST",
            path="/accounts/{accountID}/apple-pay/tokens",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.LinkApplePayTokenGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.link_apple_pay, False, False, "json", components.LinkApplePay
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="linkApplePayToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.LinkApplePayTokenResponse(
                result=unmarshal_json_response(
                    components.LinkedApplePayPaymentMethod, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.LinkApplePayErrorData, http_res
            )
            raise errors.LinkApplePayError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)

    async def link_token_async(
        self,
        *,
        account_id: str,
        token: Union[
            components.LinkApplePayToken, components.LinkApplePayTokenTypedDict
        ],
        billing_contact: Optional[
            Union[
                components.AppleBillingContact, components.AppleBillingContactTypedDict
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.LinkApplePayTokenResponse:
        r"""Connect an Apple Pay token to the specified account.

        Read our [Apple Pay tutorial](https://docs.moov.io/guides/sources/cards/apple-pay/#register-your-domains) to learn more.
        The `token` data is defined by Apple Pay and should be passed through from Apple Pay's response unmodified.

        To access this endpoint using an [access token](https://docs.moov.io/api/authentication/access-tokens/)
        you'll need to specify the `/accounts/{accountID}/cards.write` scope.

        :param account_id: ID of the Moov account representing the cardholder.
        :param token: Contains the user's payment information as returned from Apple Pay.

            Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymenttoken)
            for more information.
        :param billing_contact: Billing contact information as returned from Apple Pay.

            Refer to [Apple's documentation](https://developer.apple.com/documentation/apple_pay_on_the_web/applepaypaymentcontact)
            for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.LinkApplePayTokenRequest(
            account_id=account_id,
            link_apple_pay=components.LinkApplePay(
                token=utils.get_pydantic_model(token, components.LinkApplePayToken),
                billing_contact=utils.get_pydantic_model(
                    billing_contact, Optional[components.AppleBillingContact]
                ),
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/accounts/{accountID}/apple-pay/tokens",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=operations.LinkApplePayTokenGlobals(
                x_moov_version=self.sdk_configuration.globals.x_moov_version,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.link_apple_pay, False, False, "json", components.LinkApplePay
            ),
            allow_empty_value=None,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="linkApplePayToken",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "403",
                "404",
                "409",
                "422",
                "429",
                "4XX",
                "500",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.LinkApplePayTokenResponse(
                result=unmarshal_json_response(
                    components.LinkedApplePayPaymentMethod, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, ["400", "409"], "application/json"):
            response_data = unmarshal_json_response(errors.GenericErrorData, http_res)
            raise errors.GenericError(response_data, http_res)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = unmarshal_json_response(
                errors.LinkApplePayErrorData, http_res
            )
            raise errors.LinkApplePayError(response_data, http_res)
        if utils.match_response(http_res, ["401", "403", "404", "429"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, ["500", "504"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.APIError("API error occurred", http_res, http_res_text)

        raise errors.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from moovio_sdk.models.components import (
    account as components_account,
    capabilityid as components_capabilityid,
    capabilitystatus as components_capabilitystatus,
    createaccounttype as components_createaccounttype,
)
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
from moovio_sdk.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ListConnectedAccountsForAccountGlobalsTypedDict(TypedDict):
    x_moov_version: NotRequired[str]
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """


class ListConnectedAccountsForAccountGlobals(BaseModel):
    x_moov_version: Annotated[
        Optional[str],
        pydantic.Field(alias="X-Moov-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Specify an API version.

    API versioning follows the format `vYYYY.QQ.BB`, where
    - `YYYY` is the year
    - `QQ` is the two-digit month for the first month of the quarter (e.g., 01, 04, 07, 10)
    - `BB` is the build number, starting at `.01`, for subsequent builds in the same quarter.
    - For example, `v2024.01.00` is the initial release of the first quarter of 2024.

    The `dev` version represents the most recent development state. It may include breaking changes and should be treated as a beta release.
    When no version is specified, the API defaults to `v2024.01.00`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["X-Moov-Version"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListConnectedAccountsForAccountRequestTypedDict(TypedDict):
    account_id: str
    name: NotRequired[str]
    r"""Filter connected accounts by name.

    If provided, this query will attempt to find matches against the following Account and Profile fields:
    <ul>
    <li>Account `displayName`</li>
    <li>Individual Profile `firstName`, `middleName`, and `lastName`</li>
    <li>Business Profile `legalBusinessName`</li>
    </ul>

    Filtering by Guest Profile `name` is not currently supported.
    """
    email: NotRequired[str]
    r"""Filter connected accounts by email address.

    Provide the full email address to filter by email.
    """
    type: NotRequired[components_createaccounttype.CreateAccountType]
    r"""Filter connected accounts by AccountType.

    If the `type` parameter is used in combination with `name`, only the corresponding type's name fields will
    be searched. For example, if `type=business` and `name=moov`, the search will attempt to find matches against
    the display name and Business Profile name fields (`legalBusinessName`, and `doingBusinessAs`).

    Filtering by `type=guest` is not currently supported.
    """
    foreign_id: NotRequired[str]
    r"""Serves as an optional alias from a foreign/external system which can be used to reference this resource."""
    include_disconnected: NotRequired[bool]
    r"""Filter disconnected accounts.

    If true, the response will include disconnected accounts.
    """
    capability: NotRequired[components_capabilityid.CapabilityID]
    r"""Filter connected accounts by the capability."""
    capability_status: NotRequired[components_capabilitystatus.CapabilityStatus]
    r"""Filter connected accounts by the capability."""
    skip: NotRequired[int]
    count: NotRequired[int]


class ListConnectedAccountsForAccountRequest(BaseModel):
    account_id: Annotated[
        str,
        pydantic.Field(alias="accountID"),
        FieldMetadata(path=PathParamMetadata(style="simple", explode=False)),
    ]

    name: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter connected accounts by name.

    If provided, this query will attempt to find matches against the following Account and Profile fields:
    <ul>
    <li>Account `displayName`</li>
    <li>Individual Profile `firstName`, `middleName`, and `lastName`</li>
    <li>Business Profile `legalBusinessName`</li>
    </ul>

    Filtering by Guest Profile `name` is not currently supported.
    """

    email: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter connected accounts by email address.

    Provide the full email address to filter by email.
    """

    type: Annotated[
        Optional[components_createaccounttype.CreateAccountType],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter connected accounts by AccountType.

    If the `type` parameter is used in combination with `name`, only the corresponding type's name fields will
    be searched. For example, if `type=business` and `name=moov`, the search will attempt to find matches against
    the display name and Business Profile name fields (`legalBusinessName`, and `doingBusinessAs`).

    Filtering by `type=guest` is not currently supported.
    """

    foreign_id: Annotated[
        Optional[str],
        pydantic.Field(alias="foreignID"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Serves as an optional alias from a foreign/external system which can be used to reference this resource."""

    include_disconnected: Annotated[
        Optional[bool],
        pydantic.Field(alias="includeDisconnected"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter disconnected accounts.

    If true, the response will include disconnected accounts.
    """

    capability: Annotated[
        Optional[components_capabilityid.CapabilityID],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter connected accounts by the capability."""

    capability_status: Annotated[
        Optional[components_capabilitystatus.CapabilityStatus],
        pydantic.Field(alias="capabilityStatus"),
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Filter connected accounts by the capability."""

    skip: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    count: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(
            [
                "name",
                "email",
                "type",
                "foreignID",
                "includeDisconnected",
                "capability",
                "capabilityStatus",
                "skip",
                "count",
            ]
        )
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class ListConnectedAccountsForAccountResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: List[components_account.AccountTypedDict]


class ListConnectedAccountsForAccountResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: List[components_account.Account]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from moovio_sdk.models.errors import MoovError
from moovio_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class FileTypedDict(TypedDict):
    filename: NotRequired[str]
    mime_type: NotRequired[str]


class File(BaseModel):
    filename: Optional[str] = None

    mime_type: Annotated[Optional[str], pydantic.Field(alias="mimeType")] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["filename", "mimeType"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class FileUploadValidationErrorData(BaseModel):
    evidence_type: Annotated[Optional[str], pydantic.Field(alias="evidenceType")] = None
    file: Optional[File] = None


@dataclass(unsafe_hash=True)
class FileUploadValidationError(MoovError):
    data: FileUploadValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: FileUploadValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        message = body or raw_response.text
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)


try:
    File.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class ACHReturnCode(str, Enum):
    r"""The return code of an ACH transaction that caused the bank account status to change.

    - R02: Account Closed
    - R03: No Account/Unable to Locate Account
    - R04: Invalid Account Number
    - R05: Improper Debit to Consumer Account
    - R07: Authorization Revoked by Customer
    - R08: Payment Stopped
    - R10: Customer Advises Originator is Not Known or Authorized to Receiver
    - R11: Customer Advises Entry Not in Accordance with the Terms of the Authorization
    - R12: Branch Sold to Another DFI
    - R13: RDFI not qualified to participate
    - R14: Representative payee deceased or unable to continue in that capacity
    - R15: Beneficiary or bank account holder
    - R16: Bank account frozen
    - R17: Entry with Invalid Account Number Initiated Under Questionable Circumstances
    - R20: Non-payment bank account
    - R23: Credit entry refused by receiver
    - R29: Corporate customer advises not authorized
    - R34: Limited participation RDFI
    - R38: Stop Payment on Source Document (Adjustment Entry)
    - R39: Improper Source Document
    """

    R02 = "R02"
    R03 = "R03"
    R04 = "R04"
    R05 = "R05"
    R07 = "R07"
    R08 = "R08"
    R10 = "R10"
    R11 = "R11"
    R12 = "R12"
    R13 = "R13"
    R14 = "R14"
    R15 = "R15"
    R16 = "R16"
    R17 = "R17"
    R20 = "R20"
    R23 = "R23"
    R29 = "R29"
    R34 = "R34"
    R38 = "R38"
    R39 = "R39"
